/* Generated by wayland-scanner 1.18.0 */

#ifndef LIRI_COLOR_PICKER_SERVER_PROTOCOL_H
#define LIRI_COLOR_PICKER_SERVER_PROTOCOL_H

#include <stdint.h>
#include <stddef.h>
#include "wayland-server.h"

#ifdef  __cplusplus
extern "C" {
#endif

struct wl_client;
struct wl_resource;

/**
 * @page page_liri_color_picker The liri_color_picker protocol
 * @section page_ifaces_liri_color_picker Interfaces
 * - @subpage page_iface_liri_color_picker_manager - color picker manager
 * - @subpage page_iface_liri_color_picker - color picker
 * @section page_copyright_liri_color_picker Copyright
 * <pre>
 *
 * Copyright (C) 2020 Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * </pre>
 */
struct liri_color_picker;
struct liri_color_picker_manager;
struct wl_output;
struct wl_seat;

/**
 * @page page_iface_liri_color_picker_manager liri_color_picker_manager
 * @section page_iface_liri_color_picker_manager_desc Description
 *
 * This interface allows a client to read the ARGB value of a pixel,
 * effectively pickup up the color at a specific coordinate
 * of an output.
 * @section page_iface_liri_color_picker_manager_api API
 * See @ref iface_liri_color_picker_manager.
 */
/**
 * @defgroup iface_liri_color_picker_manager The liri_color_picker_manager interface
 *
 * This interface allows a client to read the ARGB value of a pixel,
 * effectively pickup up the color at a specific coordinate
 * of an output.
 */
extern const struct wl_interface liri_color_picker_manager_interface;
/**
 * @page page_iface_liri_color_picker liri_color_picker
 * @section page_iface_liri_color_picker_desc Description
 *
 * This object is created in response to a color picker request.
 * The picked event is emitted as soon as the operation is completed,
 * and when that happens the client has to destroy the object.
 * @section page_iface_liri_color_picker_api API
 * See @ref iface_liri_color_picker.
 */
/**
 * @defgroup iface_liri_color_picker The liri_color_picker interface
 *
 * This object is created in response to a color picker request.
 * The picked event is emitted as soon as the operation is completed,
 * and when that happens the client has to destroy the object.
 */
extern const struct wl_interface liri_color_picker_interface;

#ifndef LIRI_COLOR_PICKER_MANAGER_ERROR_ENUM
#define LIRI_COLOR_PICKER_MANAGER_ERROR_ENUM
enum liri_color_picker_manager_error {
	/**
	 * a color picker for the same output already exist
	 */
	LIRI_COLOR_PICKER_MANAGER_ERROR_ALREADY_CONSTRUCTED = 0,
	/**
	 * another pick_interactive request is still in progress
	 */
	LIRI_COLOR_PICKER_MANAGER_ERROR_STILL_IN_PROGRESS = 1,
};
#endif /* LIRI_COLOR_PICKER_MANAGER_ERROR_ENUM */

/**
 * @ingroup iface_liri_color_picker_manager
 * @struct liri_color_picker_manager_interface
 */
struct liri_color_picker_manager_interface {
	/**
	 * pick a color at a specific output location
	 *
	 * Pick the color of a pixel at specific coordinates within the
	 * bounds of the output.
	 * @param x x coordinates in output space
	 * @param y y coordinates in output space
	 */
	void (*pick_at_location)(struct wl_client *client,
				 struct wl_resource *resource,
				 uint32_t id,
				 struct wl_resource *output,
				 uint32_t x,
				 uint32_t y);
	/**
	 * pick a color interactively
	 *
	 * Start a pointer-driven pixel selection. The user moves the
	 * mouse pointer to their desired location and press a mouse
	 * button.
	 *
	 * This request fails if another interactive request is still in
	 * progress, regardless of what liri_color_picker object originated
	 * it and regardless of which output is involved.
	 * @param seat seat whose pointer is used
	 */
	void (*pick_interactively)(struct wl_client *client,
				   struct wl_resource *resource,
				   uint32_t id,
				   struct wl_resource *seat);
	/**
	 * destroy the manager
	 *
	 * All objects created by the manager will still remain valid,
	 * until their appropriate destroy request has been called.
	 */
	void (*destroy)(struct wl_client *client,
			struct wl_resource *resource);
};


/**
 * @ingroup iface_liri_color_picker_manager
 */
#define LIRI_COLOR_PICKER_MANAGER_PICK_AT_LOCATION_SINCE_VERSION 1
/**
 * @ingroup iface_liri_color_picker_manager
 */
#define LIRI_COLOR_PICKER_MANAGER_PICK_INTERACTIVELY_SINCE_VERSION 1
/**
 * @ingroup iface_liri_color_picker_manager
 */
#define LIRI_COLOR_PICKER_MANAGER_DESTROY_SINCE_VERSION 1

/**
 * @ingroup iface_liri_color_picker
 * @struct liri_color_picker_interface
 */
struct liri_color_picker_interface {
	/**
	 * release the object
	 *
	 * This destroys the liri_color_picker object.
	 */
	void (*destroy)(struct wl_client *client,
			struct wl_resource *resource);
};

#define LIRI_COLOR_PICKER_PICKED 0

/**
 * @ingroup iface_liri_color_picker
 */
#define LIRI_COLOR_PICKER_PICKED_SINCE_VERSION 1

/**
 * @ingroup iface_liri_color_picker
 */
#define LIRI_COLOR_PICKER_DESTROY_SINCE_VERSION 1

/**
 * @ingroup iface_liri_color_picker
 * Sends an picked event to the client owning the resource.
 * @param resource_ The client's resource
 * @param value ARGB color value
 */
static inline void
liri_color_picker_send_picked(struct wl_resource *resource_, uint32_t value)
{
	wl_resource_post_event(resource_, LIRI_COLOR_PICKER_PICKED, value);
}

#ifdef  __cplusplus
}
#endif

#endif
