/* Generated by wayland-scanner 1.18.0 */

#ifndef LIRI_SHELL_SERVER_PROTOCOL_H
#define LIRI_SHELL_SERVER_PROTOCOL_H

#include <stdint.h>
#include <stddef.h>
#include "wayland-server.h"

#ifdef  __cplusplus
extern "C" {
#endif

struct wl_client;
struct wl_resource;

/**
 * @page page_liri_shell The liri_shell protocol
 * @section page_ifaces_liri_shell Interfaces
 * - @subpage page_iface_liri_shell - create widgets and shell helpers
 * - @subpage page_iface_liri_osd - on-screen display
 * @section page_copyright_liri_shell Copyright
 * <pre>
 *
 * Copyright (C) 2018-2019 Pier Luigi Fiorini <pierluigi.fiorini@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * </pre>
 */
struct liri_osd;
struct liri_shell;
struct wl_surface;

/**
 * @page page_iface_liri_shell liri_shell
 * @section page_iface_liri_shell_desc Description
 *
 * This interface is bound by a special client called shell helper
 * that is responsible for helping the compositor draw its user interface.
 * It can only be bound once and only shell helper is allowed.
 * @section page_iface_liri_shell_api API
 * See @ref iface_liri_shell.
 */
/**
 * @defgroup iface_liri_shell The liri_shell interface
 *
 * This interface is bound by a special client called shell helper
 * that is responsible for helping the compositor draw its user interface.
 * It can only be bound once and only shell helper is allowed.
 */
extern const struct wl_interface liri_shell_interface;
/**
 * @page page_iface_liri_osd liri_osd
 * @section page_iface_liri_osd_desc Description
 *
 * With this interface, the compositor asks the client to display
 * either a text or a progress inside the on-screen display surface.
 * @section page_iface_liri_osd_api API
 * See @ref iface_liri_osd.
 */
/**
 * @defgroup iface_liri_osd The liri_osd interface
 *
 * With this interface, the compositor asks the client to display
 * either a text or a progress inside the on-screen display surface.
 */
extern const struct wl_interface liri_osd_interface;

#ifndef LIRI_SHELL_CURSOR_ENUM
#define LIRI_SHELL_CURSOR_ENUM
enum liri_shell_cursor {
	LIRI_SHELL_CURSOR_NONE = 0,
	LIRI_SHELL_CURSOR_RESIZE_TOP = 1,
	LIRI_SHELL_CURSOR_RESIZE_BOTTOM = 2,
	LIRI_SHELL_CURSOR_ARROW = 3,
	LIRI_SHELL_CURSOR_RESIZE_LEFT = 4,
	LIRI_SHELL_CURSOR_RESIZE_TOP_LEFT = 5,
	LIRI_SHELL_CURSOR_RESIZE_BOTTOM_LEFT = 6,
	LIRI_SHELL_CURSOR_MOVE = 7,
	LIRI_SHELL_CURSOR_RESIZE_RIGHT = 8,
	LIRI_SHELL_CURSOR_RESIZE_TOP_RIGHT = 9,
	LIRI_SHELL_CURSOR_RESIZE_BOTTOM_RIGHT = 10,
	LIRI_SHELL_CURSOR_BUSY = 11,
};
#endif /* LIRI_SHELL_CURSOR_ENUM */

/**
 * @ingroup iface_liri_shell
 * @struct liri_shell_interface
 */
struct liri_shell_interface {
	/**
	 * set grab surface
	 *
	 * The surface set by this request will receive a fake
	 * pointer.enter event at position 0, 0 and is expected to set an
	 * appropriate cursor image every time the grab_cursor event is
	 * sent by the compositor.
	 */
	void (*set_grab_surface)(struct wl_client *client,
				 struct wl_resource *resource,
				 struct wl_resource *surface);
	/**
	 * shell is ready
	 *
	 * Tell the compositor that the shell has created all the layer
	 * surfaces and it can hide the splash screen revealing the
	 * desktop.
	 */
	void (*ready)(struct wl_client *client,
		      struct wl_resource *resource);
};

#define LIRI_SHELL_GRAB_CURSOR 0
#define LIRI_SHELL_QUIT 1

/**
 * @ingroup iface_liri_shell
 */
#define LIRI_SHELL_GRAB_CURSOR_SINCE_VERSION 1
/**
 * @ingroup iface_liri_shell
 */
#define LIRI_SHELL_QUIT_SINCE_VERSION 1

/**
 * @ingroup iface_liri_shell
 */
#define LIRI_SHELL_SET_GRAB_SURFACE_SINCE_VERSION 1
/**
 * @ingroup iface_liri_shell
 */
#define LIRI_SHELL_READY_SINCE_VERSION 1

/**
 * @ingroup iface_liri_shell
 * Sends an grab_cursor event to the client owning the resource.
 * @param resource_ The client's resource
 */
static inline void
liri_shell_send_grab_cursor(struct wl_resource *resource_, uint32_t cursor)
{
	wl_resource_post_event(resource_, LIRI_SHELL_GRAB_CURSOR, cursor);
}

/**
 * @ingroup iface_liri_shell
 * Sends an quit event to the client owning the resource.
 * @param resource_ The client's resource
 */
static inline void
liri_shell_send_quit(struct wl_resource *resource_)
{
	wl_resource_post_event(resource_, LIRI_SHELL_QUIT);
}

#define LIRI_OSD_TEXT 0
#define LIRI_OSD_PROGRESS 1

/**
 * @ingroup iface_liri_osd
 */
#define LIRI_OSD_TEXT_SINCE_VERSION 1
/**
 * @ingroup iface_liri_osd
 */
#define LIRI_OSD_PROGRESS_SINCE_VERSION 1


/**
 * @ingroup iface_liri_osd
 * Sends an text event to the client owning the resource.
 * @param resource_ The client's resource
 */
static inline void
liri_osd_send_text(struct wl_resource *resource_, const char *icon_name, const char *text)
{
	wl_resource_post_event(resource_, LIRI_OSD_TEXT, icon_name, text);
}

/**
 * @ingroup iface_liri_osd
 * Sends an progress event to the client owning the resource.
 * @param resource_ The client's resource
 */
static inline void
liri_osd_send_progress(struct wl_resource *resource_, const char *icon_name, uint32_t value)
{
	wl_resource_post_event(resource_, LIRI_OSD_PROGRESS, icon_name, value);
}

#ifdef  __cplusplus
}
#endif

#endif
